# ip232.tcl---RS232 IP pack diagnostic frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame ip232
set frametitle "IP 232 Diagnostics Frame"
set icontitle "ip232"

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource VME}

set mname "IP232"

set VIPC616BaseAddress 0xffffc000
set IPslot 2
set IPchannel 0

set rname ""
set command ""
set response ""

set paritymodechoices [list "With parity" "Force parity" "No parity" "Special mode"]
set paritytypechoices [list Even Odd]
set databitschoices   [list 5 6 7 8]
set stopbitschoices   [list 0.563 0.625 0.688 0.750 0.813 0.875 0.938 1.000 \
  1.563 1.625 1.688 1.750 1.813 1.875 1.938 2.000]
set baudratechoices   [list 50 110 134.5 200 300 600 1200 1050 2400 4800 7200 9600 38.4K]

#    options
#   baud rate 11 = 9600; no parity;7 bits per character; 1 stop bits

set baudrate 11
set paritymode 2
set parity 0
set databits 2
set stopbits 7

#    values written to the IP232
set mr1 0
set mr2 0
set csr 0

#    define tables for text => hex & hex => text conversion

set v [expr 0x41]
foreach c [split ABCDEFGHIJKLMNOPQRSTUVWXYZ ""] {
  set 2hex($c) $v
  set 2char($v) $c
  incr v
}
set v [expr 0x61]
foreach c [split abcdefghijklmnopqrstuvwxyz ""] {
  set 2hex($c) $v
  set 2char($v) $c
  incr v
}
set v [expr 0x30]
foreach c [split 0123456789 ""] {
  set 2hex($c) $v
  set 2char($v) $c
  incr v
}
set 2hex(\ ) 0x20
set 2char(32) " "



proc define-register {class name attributes} {
   global resource mname
   set register /$resource/$mname.$name
   set-footer "defining register $register"
   catch {EG "register :name '$register' :class '$class' !new"}
   if {$attributes != ""} then {EG "register :name '$register' :object '$attributes'"}
   clear-footer
}

proc write-register {name value} {
   global testmode resource mname
   global monitor
   set register /$resource/$mname.$name
   if {$monitor != 0} then {
      if {$name == "TxCMD" || $name == "CMD"} then {set-footer "writing to $register"} \
         else {set-footer "writing $value to $register"}
   }
   if {$name == "MR1"} then {
#    to access the MR1 register must first reset the MR pointer via the CR 
      EG "register :name '/$resource/$mname.CR' :value '16'"
      if {$testmode == 1} then {insert-debug "writing 0x10 to /$resource/$mname.CR"}
   }
   EG "register :name '$register' :value '$value'"
   if {$testmode == 1} then {insert-debug "writing $value to $register"}
   if {$monitor != 0} then {clear-footer}
}

proc read-register {name} {
   global testmode resource mname
   global monitor
   set register /$resource/$mname.$name
   if {$monitor != 0} then {set-footer "reading $register"}
   if {$name == "MR1"} then {
#    to access the MR1 register must first reset the MR pointer via the CR 
      EG "register :name '/$resource/$mname.CR' :value '16'"
      if {$testmode == 1} then {insert-debug "writing 0x10 to /$resource/$mname.CR"}
   }
   set value [EG "register :name '$register' ?value"]
   if {$testmode == 1} then {insert-debug "$register returned $value"}
   if {$monitor != 0} then {clear-footer}
   return $value
}

proc initialise-register {name} {
   global testmode resource mname
   global monitor
   set register /$resource/$mname.$name
   if {$monitor != 0} then {set-footer "initialising $register"}
   EG "register :name '$register' !initialise"
   if {$testmode == 1} then {insert-debug "initialising $register"}
   if {$monitor != 0} then {clear-footer}
}

proc rr {name} {read-register $name}
proc wr {name value} {write-register $name $value}

proc define-module {} {
  global resource mname IPslot IPchannel VIPC616BaseAddress
  set-footer "deleting all IP232 registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

#    Mode Register 1
  define-register xIP232R MR1        "-a $VIPC616BaseAddress -o 1 -s $IPslot -c $IPchannel -p rw"

#    Mode Register 2
  define-register xIP232R MR2        "-a $VIPC616BaseAddress -o 1 -s $IPslot -c $IPchannel -p rw"

#    Status Register
  define-register xIP232R SR         "-a $VIPC616BaseAddress -o 3 -s $IPslot -c $IPchannel -p ro"

#    Clock Select Register
  define-register xIP232R CSR        "-a $VIPC616BaseAddress -o 3 -s $IPslot -c $IPchannel -p wo"

#    Command Register
  define-register xIP232R CR         "-a $VIPC616BaseAddress -o 5 -s $IPslot -c $IPchannel -p wo"

#    Rx & Tx Holding Registers
  define-register xIP232R RHR        "-a $VIPC616BaseAddress -o 7 -s $IPslot -c $IPchannel -p ro"
  define-register xIP232R THR        "-a $VIPC616BaseAddress -o 7 -s $IPslot -c $IPchannel -p wo"

#    command (without response)
  define-register xIP232C TxCMD  "-a $VIPC616BaseAddress -s $IPslot -c $IPchannel -p wo"

#    command (with response)
  define-register xIP232C CMD    "-a $VIPC616BaseAddress -s $IPslot -c $IPchannel -p rw"

#    Update IP details

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == 0xf0 && $modid == 0xaa} then {set passed 1} else {set passed 0}
     update-widget PASSED $passed

     click-redisplay
}

#    functions available as commands via the CR
proc reset-Rx {}          {write-register CR 0x20}
proc reset-Tx {}          {write-register CR 0x30}
proc reset-ErrStatus {}   {write-register CR 0x40}
proc enable-Tx {}         {write-register CR 0x84}
proc enable-Rx {}         {write-register CR 0x01}
proc disable-Tx {}        {write-register CR 0x98}
proc disable-Rx {}        {write-register CR 0x02}
proc start-break {}       {write-register CR 0x60}
proc stop-break {}        {write-register CR 0x70}


proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-setup {} {
  global baudrate paritymode parity databits stopbits
  global mr1 mr2 csr
#       Mode Register 1  (MR1)
     set mr1 [expr [expr [expr $paritymode << 1] + $parity << 2] + $databits]
     write-register MR1 $mr1
#       Mode Register 2  (MR2)
     set mr2 $stopbits
     write-register MR2 $mr2
#       Clock Select Register (CSR)
     set csr [expr [expr $baudrate << 4] + $baudrate]
     write-register CSR $csr
}

proc click-reset {} {
#    perform hardware reset
      reset-Tx
      reset-Rx
      reset-ErrStatus
#    and re-setup
   click-setup
}

proc Exec1 {} {
    global command
    Tx-command "$command"
    initialise-register TxCMD
}

proc Exec2 {} {
    global command response
    Tx-request "$command"
    initialise-register CMD
    set response [read-register CMD]
    :modify REPLY :value "$response"
}

proc Tx-command {s} {
#    send a string of characters (command) - no response is expected
   :modify CMD :value "$s"
   :modify REPLY :value ""
   update idletasks
   write-register TxCMD "$s"
}

proc Tx-request {s} {
#    send a string of characters (command) - response is expected
   :modify CMD :value "$s"
   :modify REPLY :value ""
   update idletasks
   write-register CMD "$s"
}

proc Rx-response {} {
#   pick up response from previous command request
   set data [read-register CMD]
   :modify REPLY :value "$data"
   return $data
}

proc Check-Rx {} {
  global sr
  set sr [read-register SR]
  if {[expr $sr & 0xf0] != 0} then {midas-warning "Rx Status Register is $sr"}
}

proc click-command {s} {
  global command
  set command "$s"
}

proc click-redisplay {} {
  global sr
#    display  SR
     set sr [read-register SR]
     foreach bit {7 6 5 4 3 2 1 0} {update-widget SR$bit 0}
     foreach bit [bits-in $sr] {update-widget SR$bit 1}
}

proc click arg {
   global debug
  global VIPC616BaseAddress IPslot IPchannel
  global paritymodechoices paritymode
  global paritytypechoices parity
  global databitschoices databits
  global stopbitschoices stopbits
  global baudratechoices baudrate

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }
      IPCHANNEL {
          set IPchannel $v
          define-module
      }

      PARITYMODE {set paritymode $v}
      PARITY     {set parity $v}
      DATABITS   {set databits $v}
      STOPBITS   {set stopbits $v}
      BAUDRATE   {set baudrate $v}

      CMD       {click-command $v}

      default   {puts stdout $arg}
   }
}



proc make-frame {} {
  global env VIPC616BaseAddress IPslot IPchannel
  global paritymodechoices paritymode
  global paritytypechoices parity
  global databitschoices databits
  global stopbitschoices stopbits
  global baudratechoices baudrate


  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 20

      :hskip 50
      :text IPBASEADDR :label "IP Carrier Base Address:" :value $VIPC616BaseAddress
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D]         :value $IPslot
      :choice IPCHANNEL  :label "Channel"      :strings [list 0 1 2 3 4 5 6 7] :value $IPchannel
      :next-row T
      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 15
      :choice-stack PARITYMODE :label "Parity Mode" :strings $paritymodechoices  \
          :value $paritymode
      :choice-stack PARITY     :label "Parity Type" :strings $paritytypechoices \
          :value $parity
      :choice-stack DATABITS   :label "Bits per Character" :strings $databitschoices \
          :value $databits
      :choice-stack STOPBITS   :label "Stop Bit Length" :strings $stopbitschoices \
          :value $stopbits
      :choice-stack BAUDRATE   :label "Baud Rate" :strings $baudratechoices \
          :value $baudrate
     :next-row T
      :button REDISPLAY :label "Redisplay"              :command "click-redisplay"
      :button DEFINE    :label "Define IP232 registers" :command "define-module"
      :button RESET     :label "Reset IP232"            :command "click-reset"
      :button SETUP     :label "Setup IP232"            :command "click-setup"
   
   :panel STATUS  :layout T :padx 10
      :layout H; :row-gap 5; :col-gap 15
       :message M1 :label "SR (Status Register)" :bold T
       :next-row T
       :checkbox SR7 :deaf T :label "Rx Break"
       :checkbox SR6 :deaf T :label "Frame Err"
       :checkbox SR5 :deaf T :label "Parity Err"
       :checkbox SR4 :deaf T :label "Overrun Err"
       :checkbox SR3 :deaf T :label "Tx Empty"
       :checkbox SR2 :deaf T :label "Tx Ready"
       :checkbox SR1 :deaf T :label "Fifo Full"
       :checkbox SR0 :deaf T :label "Rx ready"

   :panel TEST  :layout T :padx 10 :pady 20
      :layout H; :row-gap 15; :col-gap 15
      :text CMD    :label "Enter a command" :w 32
      :button EXEC1   :label "command only"          :command "Exec1"
      :button EXEC2   :label "command + response"    :command "Exec2"
      :text REPLY  :label "Response was"    :w 32  :deaf T
   
   :show T

}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
#      define-module
#      click-reset
   } else {open-frame}
}

