# V550.tcl---CAEN V550 C-RAMS module control frame

set frame "V550"
set frametitle "CAEN V550 2 Channel C-RAMS"
set icontitle "V550"

set modulename v550
set moduletype  "CAEN V550"

loadsource common.vme

set allmodules 0


set w2r(NUMCHAN0) Ch0.NumChan
set w2r(NUMCHAN1) Ch1.NumChan
set w2r(T)       Status.TM
set w2r(MO)      Status.MO
set w2r(/D0)     Status.Ch0.DRDY
set w2r(/D1)     Status.Ch1.DRDY
set w2r(/E0)     Status.Ch0.FMTY
set w2r(/E1)     Status.Ch1.FMTY
set w2r(/H0)     Status.Ch0.FHF
set w2r(/H1)     Status.Ch1.FHF
set w2r(/F0)     Status.Ch0.FFULL
set w2r(/F1)     Status.Ch1.FFULL
set w2r(TP0)     Ch0.TestPatt
set w2r(TP1)     Ch1.TestPatt
set w2r(WC0)     Ch0.WordCount
set w2r(WC1)     Ch1.WordCount
set w2r(FIFO0)   Ch0.FIFO
set w2r(FIFO1)   Ch1.FIFO
set w2r(CLEAR)   Clear
set w2r(STATUS)  Status
set w2r(T0)      Ch0.Threshold\[0-2047\]
set w2r(T1)      Ch1.Threshold\[0-2047\]
set w2r(P0)      Ch0.Pedestal\[0-2047\]
set w2r(P1)      Ch1.Pedestal\[0-2047\]



proc click-widget {w value} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-registers $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc update-widget {w} {
    global w2r modulename module
    
    set register $w2r($w)
    set value [read-vme-register $modulename#$module.$register]
    :modify $w :value $value
}

proc click-select-module {v} {
   global module maxmodule
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc click-ID {} {
    global modulename module
    foreach id {1 2 3} {
       set vl [read-vme-register $modulename#$module.ModID#$id]
       insert-log "V550 Module Identifier Word $id = $vl"
    }
}
   
proc redisplay-module {} {
   global w2r modulename module maxmodule

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   foreach w {STATUS TP0 TP1 WC0 WC1} {update-widget $w}

   foreach w {NUMCHAN0 NUMCHAN1} {
       set register $w2r($w)
       set value [read-vme-register $modulename#$module.$register]
       :modify $w :value [expr $value * 32]
   }

#    thresholds and pedestals
#    there are 2048 of these!!!
#    just show the first!!

   set register Ch0.Threshold0
   :modify T0 :value [read-vme-register $modulename#$module.$register]
   set register Ch1.Threshold0
   :modify T1 :value [read-vme-register $modulename#$module.$register]
   set register Ch0.Pedestal0
   :modify P0 :value [read-vme-register $modulename#$module.$register]
   set register Ch1.Pedestal0
   :modify P1 :value [read-vme-register $modulename#$module.$register]
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module

   clear-busy
   set EGreport 0
}

proc click-registers {} {
   global EGreport moduletype modulename module registers_table

   if {[info procs [set moduletype]_registers] == ""} then  \
        {set-footer "No [set moduletype]_registers procedure"; return}

   [set moduletype]_registers

   clear-footer
   set EGreport -1

   set info "[set moduletype] - module $modulename#$module\n\n"

   foreach item $registers_table {
       set description [first $item]
       set register [second $item]
       set data [read-vme-register $modulename#$module.$register]
       append info "[format "%-40s %-20s" $description $data]\n"
   }

  midas-information $info

   set EGreport 0
}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}


proc click arg {
   global debug
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      
      STATUS    {click-widget $w [expr $v & 3]}
      NUMCHAN*  {click-widget $w [expr ($v + 31) * 32]}
      
      CLEAR     {click-widget $w 0}
      
      FIFOR0    {update-widget FIFO0}
      FIFOR1    {update-widget FIFO1}
      
      REDISPLAY      {click-redisplay}
      ID             {click-ID}
      
      P0   -
      P1   -
      T0   -
      T1        {click-widget $w $v}
      
      ENGINEERING       {click-$v}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env
  global module maxmodule modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :hskip 200
     :checkbox ALL-MODULES :label "Act on ALL modules?"
     
     :next-row T
     :message M1 :label "Status Register" :bold T
     :choice STATUS :label "" :strings [list T MO /D0 /D1 /E0 /E1 /H0 /H1 /F0 /F1] :w 3 :exclusive F

     :next-row T
     :text NUMCHAN0 :label "Number of detector channels Ch0" :w 6 :valuex 32c
     :text NUMCHAN1 :label "Number of detector channels Ch1" :w 6 :valuex 32c
     
     :next-row T
     :text TP0 :label "Test pattern Ch0" :w 8 :valuex 32c
     :text TP1 :label "Test pattern Ch1" :w 8 :valuex 28c
     
     :next-row T
     :slider T0 :label "Ch0 thresholds" :min 0 :max 255 :valuex 15c
     :slider T1 :label "Ch1 thresholds" :min 0 :max 255 :valuex 15c
     
     :next-row T
     :slider P0 :label "Ch0 pedestals"  :min 0 :max 255 :valuex 15c
     :slider P1 :label "Ch1 pedestals"  :min 0 :max 255 :valuex 15c
     
     :next-row T
     :text WC0   :label "Word Counter Ch0" :deaf T :w 6 :valuex 15c
     :text FIFO0 :label "FIFO Ch0" :deaf T :w 12 :valuex 15c
     :button FIFOR0 :label "Read"
     
     :next-row T
     :text WC1   :label "Word Counter Ch1" :deaf T :w 6 :valuex 15c
     :text FIFO1 :label "FIFO Ch1" :deaf T :w 12 :valuex 15c
     :button FIFOR1 :label "Read" 
  
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :button CLEAR    :label "Generate CLEAR"
      :button ID       :label "Identify Module"
      :menubutton ENGINEERING :label "Engineering"  :menu [list software registers]

  :show T

}
