
#    vmetestcard.tcl    control for testing VME module

set frame TESTCARD
set title "Test VME Module Setup and Control"
set icontitle TestCard

set defaultmodulelist [list 1 2 3 4 5 6 7 8]

set modulelist {???}
set moduleindex 0
set module [lindex $modulelist $moduleindex]

#    for example v785
set modulename Unknown

#    for example CAEN V785
set moduletype  Unknown

set moduletypewidget text

set CurrentVMEModule Unknown
set VMEaddressdata 0x0

set registerclasses [list xVME]
set registerclass 1

set registerbase ""

set registers {}
set currentregister ""
set registerindex -1
set registeraction 0
set registerdata ""

loadsource common.vme


proc {bus-error} {module} {

   display-error-report self "Bus error accessing $module - module probably missing or broken\n" \
        "Check the VME server and check that the VME address Info supplied in the configuration is correct"
}

proc make-modulelist {} {
    global modulelist defaultmodulelist
    set modulelist $defaultmodulelist
}

proc make-myserver {} {
   global env CurrentVMEServer

   if {[info exists env(VmeServer)] == 0} then {set resource vme}  \
      else {set resource $env(VmeServer)}

   set CurrentVMEServer $resource
}

#    Note - define acts on the whole module (VME card)
proc click_define {} {
   global modulename moduletype EGreport CurrentVMEServer CurrentVMEModule VMEaddressdata

   loadsource [set modulename].defn

   set-footer "defining VME module $CurrentVMEModule"
   set-busy
   set EGreport -1
   Add_[set moduletype]_Module $CurrentVMEModule $CurrentVMEServer $VMEaddressdata
   set EGreport 0
   clear-busy
   clear-footer
}

#    Note - initialise acts on the whole module (VME card)
proc click_initialise {} {
   global CurrentVMEModule
   initialise-vme-registers  "$CurrentVMEModule.*"
}


#    Note - reset, halt and go act on the whole module (VME card)
proc DAQ-action {procname} {
   global moduletype CurrentVMEModule CurrentVMEServer
   if {[info procs [set moduletype]$procname] != ""} then {[set moduletype]$procname $CurrentVMEModule $CurrentVMEServer} \
       else {set-footer "No [set moduletype]$procname procedure"}
}

proc click_reset {} {DAQ-action "_DAQ-reset"}
proc click_halt {}  {DAQ-action "_DAQ-halt"}
proc click_go {}    {DAQ-action "_DAQ-go"}
proc click_setup {} {DAQ-action "_DAQ-setup"}

proc repaint-module {} {
   global moduletype modulename moduletypewidget modulelist moduleindex CurrentVMEServer VMEaddressdata

   if {$moduletypewidget == "text"} then {
      :modify CARDTYPE :value $moduletype
   } else {
      set modulenames [inform vmecontrol inquire modulenames]
      :modify CARDTYPE :value [lsearch $modulenames $modulename]
   }

   :modify CHANNEL :strings $modulelist :value $moduleindex
   :modify SERVER  :value $CurrentVMEServer
   :modify ADDRDATA :value $VMEaddressdata
}

proc click_vmeregisters {} {
   global CurrentVMEModule
   set data [read-vme-registers $CurrentVMEModule.VME.*]
   set info ""
   foreach item $data {
       append info "[format "%30s %20s" [first $item] [second $item]]\n"
   }
   midas-information $info
}

proc click_updateregisters {} {
   global env CurrentVMEModule
   global registers
   set data [read-vme-attributes $CurrentVMEModule.*]
 if {[info exists env(ServerAccessVersion)] && $env(ServerAccessVersion) == 2} then {set data [first $data]}
   set registers {}
   foreach item $data {
       lappend registers "[format "%30s %100s" [first $item] [second $item]]"
   }
   :modify REGL :append F :strings $registers :see 0
}

proc click_editregisters {} {
   global env registeraction registerclasses registerclass

   :frame [self frame-name].registers :label  "VME Registers for [self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]"

     :panel REGP1 :layout T
       :list REGL :rows 30  :cols 132 :exclusive T :scroll B
     :panel REGP2 :layout B :padx 10 :pady 10
      :layout H; :col-gap 20; :row-gap 5
       :text REGREG  :label "selected register" :w 120 :valuex 20c :shifters T
      :next-row T
       :text REGDATA  :label "register value"    :w 120 :valuex 20c
      :next-row T
       :button REGNEW    :label "define register"  :command click_defineregister
       :button REGDELETE :label "delete register"  :command click_deleteregister
       :button REGMODIFY :label "modify register"  :command click_modifyregister
       :checkbox REGACTION :label "apply to module"   :value $registeraction
       :button REGUPDATE   :label "update list from module"  :command click_updateregisters
       :choice-stack REGCLASS  :label "define using class" :menu $registerclasses :value $registerclass
      :next-row T
       :button REGREAD   :label "read register"    :command click_readregister
       :button REGWRITE  :label "write register"   :command click_writeregister
       :button REGINIT   :label "initialise register"    :command click_initialiseregister
       :button REGSAVE   :label "save"  :inactive T
       :hskip 50
       :checkbox VMETRACEMODE :label "VME crate TraceMode"

   :show T
   click_updateregisters
}

proc click_selectregister {v i} {
  global currentregister registerindex registerdata
   set currentregister "[first $v]   [tail $v]"
   set registerindex $i
   :modify REGREG :value $currentregister
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_editregister {v} {
  global currentregister
  set currentregister $v
}

proc click_deleteregister {} {
   global currentregister registerindex registerdata registers registeraction
   set registers [lreplace $registers $registerindex $registerindex]
   :modify REGL :append F :strings $registers :see 0
   if {$registeraction == 1} then {delete-vme-register [first $currentregister]}
   set registerindex -1
   set currentregister ""
   :modify REGREG :value $currentregister
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_defineregister {} {
   global currentregister registerindex registerdata registers registeraction
   global registerclasses registerclass
   set r [format "%30s %100s" [first $currentregister] [tail $currentregister]]
   set registers [lappend registers $r]
   :modify REGL :append F :strings $registers
   if {$registeraction == 1} then {
      set r [split [first $currentregister] .]
      set mname [first $r]
      set rname [join [tail $r] .]
      set class [lindex $registerclasses $registerclass]
      set attributes [tail $currentregister]
      define-vme-register $mname $class $rname $attributes
   }
   set registerindex [expr [llength $registers] -1]
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_modifyregister {} {
   global currentregister registerindex registerdata registers registeraction
   set r [format "%30s %100s" [first $currentregister] [tail $currentregister]]
   set registers [lreplace $registers $registerindex $registerindex $r]
   :modify REGL :append F :strings $registers :see $registerindex
   if {$registeraction == 1} then {write-vme-attribute [first $currentregister] [tail $currentregister]}
   set registerdata ""
   :modify REGDATA :value $registerdata
}

proc click_readregister {} {
   global currentregister registerdata
   set registerdata [read-vme-register [first $currentregister]]
   :modify REGDATA :value $registerdata
}

proc click_writeregister {} {
   global currentregister registerdata
   write-vme-register [first $currentregister] $registerdata
   set registerdata [read-vme-register [first $currentregister]]
   :modify REGDATA :value $registerdata
}

proc click_registerdata {v} {
   global  registerdata
   set registerdata $v
}
   
proc click_initialiseregister {} {
   global currentregister registerdata
   initialise-vme-register [first $currentregister]
   set registerdata [read-vme-register [first $currentregister]]
   :modify REGDATA :value $registerdata
}

proc click_registers {} {
   global EGreport EGallow moduletype CurrentVMEModule registers_table

   if {[info procs [set moduletype]_registers] != ""} then {[set moduletype]_registers} \
       else {set-footer "No [set moduletype]_registers procedure"; return}

   clear-footer
   set EGreport -1

   set info "[set moduletype] - module $CurrentVMEModule\n\n"

   set EGallow 0x2000a

   foreach item $registers_table {
       set description [first $item]
       set register [second $item]
       set data [read-vme-register $CurrentVMEModule.$register]
       if {$data == $EGallow} then {set data "Bus Error"}
       append info "[format "%-40s %-20s" $description $data]\n"
   }

   set EGallow 0xffffffff

   global _information_font
   set _information_font fixed
   midas-information $info

   set EGreport 0
}

proc click arg {
   global debug registeraction registerclass vmeslot
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {

      CHANNEL     {click_module $v}
      CARDTYPE    {click_moduletype $v}

      REGL        {click_selectregister $v $i}
      REGREG      {click_editregister $v}
      REGDATA     {click_registerdata $v}
      REGACTION   {set registeraction $v}
      REGCLASS    {set registerclass $v}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click_moduletype {v} {
   global modulename
   set modulenames [inform vmecontrol inquire modulenames]
   set modulename [lindex $modulenames $v]
   loadsource [set modulename].defn
   start
}

proc click_module v {
   global modulename moduleindex modulelist module CurrentVMEModule VMEaddressdata
   set moduleindex $v
   set module [lindex $modulelist $moduleindex]
   set CurrentVMEModule $modulename#$module
   set VMEaddressdata 0
   make-myserver
   repaint-module
}

proc click_addrdata {args} {
   global VMEaddressdata
   set VMEaddressdata [second $args]
}

proc click_server {args} {
   global CurrentVMEServer
   set CurrentVMEServer [second $args]
}


proc make-frame {} {
  global env moduletype moduletypewidget

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :choice-stack CHANNEL :label "Module:" :strings [list ???] 
      if {$moduletype != "Unknown"} then {
         :text CARDTYPE :label "Card Type" :w 16 :deaf T :value $moduletype
         set moduletypewidget  "text"
      } else {
         :choice-stack CARDTYPE :label "Card Type" :strings [list ???] :value 0
         set moduletypewidget  "choice"
      }

     :next-row T
      :text  SERVER   :label "VME Server"       :w 8  :valuex 16c :command click_server 
     :next-row T
      :text  ADDRDATA :label "VME Address Info" :w 40 :valuex 16c :command click_addrdata 

     :next-row T
      :button B3  :label "Access and edit module registers"   :command click_editregisters
      :button B4  :label "Display register values"     :command click_registers

     :next-row T
      :button DEFINE :label "Define module registers"        :command click_define
      :button INIT   :label "Initialise  module registers"   :command click_initialise

     :next-row T
      :col-gap 20
      :button RESET  :label RESET  :w 6 :command click_reset
      :button SETUP  :label SETUP  :w 6 :command click_setup
      :button GO     :label GO     :w 6 :command click_go
      :button HALT   :label HALT   :w 6 :command click_halt

 :show T
}

proc start args {
   global started modulename CurrentVMEModule
   global allchans
   global modulelist defaultmodulelist moduleindex module
   if {$started != 0} then {open-frame} else {
	set allchans 0
	make-frame
        set started 1
   }
   set-busy
   if {$modulename != "Unknown"} then {
      make-modulelist
      set moduleindex 0
      set module [lindex $modulelist $moduleindex]
      set CurrentVMEModule $modulename#$module
      make-myserver
      repaint-module
   } else {
      set modulelist $defaultmodulelist
      set moduleindex 0
      :modify CHANNEL :strings $modulelist :value $moduleindex
      :modify CARDTYPE :strings [inform vmecontrol inquire modulechoice] :value 0
      set module [lindex $modulelist $moduleindex]
   }
   clear-busy
}

